package im_chat_room_user

import (
	"context"
	"im-microservice/db"
	"im-microservice/helper"
	"im-microservice/pb"

	"google.golang.org/grpc/status"
)

type ChatRoomUserServer struct {
	pb.UnimplementedChatRoomUserServer
}

type RoomUserOperator struct {
	RoomId   int64  `validate:"required"`
	Operator string `validate:"required"`
}

type ChatRoomUser struct {
	RoomId int64  `validate:"required"`
	Accid  string `validate:"required"`
}

type ChatRoomUserRole struct {
	RoomId   int64  `validate:"required"`
	Operator string `validate:"required"`
	Accid    string `validate:"required"`
	Role     int64  `validate:"required,numeric"`
}

//添加管理员
func (c *ChatRoomUserServer) AddOperator(ctx context.Context, in *pb.RoomUserAddOpeRequest) (reply *pb.RoomUserReply, err error) {
	roomUserOperator := &RoomUserOperator{
		RoomId:   in.GetRoomId(),
		Operator: in.GetOperator(),
	}
	//验证参数
	if err = helper.Valiator(roomUserOperator); err != nil {
		return
	}
	//查询房间是否已经添加过了
	if r_user, _ := GetChatRoomUserOper(roomUserOperator.RoomId, roomUserOperator.Operator); r_user.Operator != "" {
		err = status.Error(20438, "该聊天室已存在管理员")
		return
	}

	//添加管理员
	r_user, err := AddChatRoomUser(roomUserOperator.RoomId, roomUserOperator.Operator, true)
	if err != nil {
		err = status.Error(500, "添加管理员失败")
		return
	}
	reply = &pb.RoomUserReply{}
	reply.RoomId = r_user.RoomId
	reply.Operator = r_user.Operator
	reply.Accid = r_user.Accid
	reply.Role = r_user.Role
	return
}

//添加普通用户
func (c *ChatRoomUserServer) ChatRoomUserAdd(ctx context.Context, in *pb.RoomUserAddRequest) (reply *pb.RoomUserReply, err error) {
	chatRoomUser := &ChatRoomUser{
		RoomId: in.GetRoomId(),
		Accid:  in.GetAccid(),
	}
	//验证参数
	if err = helper.Valiator(chatRoomUser); err != nil {
		return
	}
	//查询是否已经有了
	if r_user, _ := GetChatRoomUser(chatRoomUser.RoomId, chatRoomUser.Accid); r_user.Accid != "" {
		err = status.Error(20438, "聊天室已存在该用户")
		return
	}

	//添加用户
	r_user, err := AddChatRoomUser(chatRoomUser.RoomId, chatRoomUser.Accid, false)
	if err != nil {
		err = status.Error(500, "聊天室添加用户失败")
		return
	}
	reply = &pb.RoomUserReply{}
	reply.RoomId = r_user.RoomId
	reply.Accid = r_user.Accid
	reply.Role = r_user.Role
	return
}

//设置用户角色
func (c *ChatRoomUserServer) ChatRoomUserSetRole(ctx context.Context, in *pb.RoomUserSetRoleRequest) (reply *pb.RoomUserReply, err error) {
	setRole := &ChatRoomUserRole{
		RoomId:   in.GetRoomId(),
		Operator: in.GetOperator(),
		Accid:    in.GetAccid(),
		Role:     in.GetRole(),
	}
	if err = helper.Valiator(setRole); err != nil {
		return
	}
	if setRole.Role > 4 || setRole.Role < 1 {
		err = status.Error(401, "聊天权限超出异常")
		return
	}
	//查看是否该房间操作者
	_, err = GetChatRoomUserOper(setRole.RoomId, setRole.Operator)
	if err != nil {
		err = status.Error(403, "不是该聊天室操作者")
		return
	}

	//查询该用户是否存在
	_, err = GetChatRoomUser(setRole.RoomId, setRole.Accid)
	if err != nil {
		err = status.Error(404, "不是该聊天室用户")
		return
	}

	//设置权限
	r_user, err := SetChatRoomUser(setRole.RoomId, setRole.Accid, setRole.Role)
	if err != nil {
		return
	}
	reply = &pb.RoomUserReply{}
	reply.RoomId = r_user.RoomId
	reply.Operator = r_user.Operator
	reply.Accid = r_user.Accid
	reply.Role = r_user.Role
	return
}

//获取单个用户
func (c *ChatRoomUserServer) ChatRoomUserOne(ctx context.Context, in *pb.RoomUserOneRequest) (reply *pb.RoomUserOneReply, err error) {
	chatRoomUser := &ChatRoomUser{
		RoomId: in.GetRoomId(),
		Accid:  in.GetAccid(),
	}
	//验证参数
	if err = helper.Valiator(chatRoomUser); err != nil {
		return
	}
	c_user, err := GetChatRoomUser(chatRoomUser.RoomId, chatRoomUser.Accid)
	if err != nil {
		return
	}
	reply = &pb.RoomUserOneReply{}
	reply = getOneData(c_user)
	return
}

func getOneData(c_user db.ImChatRoomUser) *pb.RoomUserOneReply {
	return &pb.RoomUserOneReply{
		Id:         int64(c_user.Id),
		RoomId:     int64(c_user.RoomId),
		Operator:   c_user.Operator,
		Accid:      c_user.Accid,
		Role:       c_user.Role,
		Createtime: c_user.Createtime,
		Updatetime: c_user.Updatetime,
	}
}

//获取房间用户列表
func (c *ChatRoomUserServer) ChatRoomUserList(ctx context.Context, in *pb.RoomUserListRequest) (reply *pb.RoomUserListReply, err error) {
	results, err := GetChatRoomUserList(in)
	if err != nil {
		return
	}

	reply = &pb.RoomUserListReply{Paginate: &pb.Page{}}

	c_user_list := results["list"].([]db.ImChatRoomUser)
	for _, v := range c_user_list {
		reply.List = append(reply.List, getOneData(v))
	}

	reply.Paginate.PageSize = int64(results["page_size"].(int))
	reply.Paginate.PagetNo = results["page"].(int64)
	reply.Paginate.TotalCount = results["total_count"].(int64)
	reply.Paginate.TotalPage = results["total_page"].(int64)
	return
}

//删除
func (c *ChatRoomUserServer) ChatRoomUserDel(ctx context.Context, in *pb.RoomUserOneRequest) (reply *pb.RoomUserDelReply, err error) {
	delRoomUser := &ChatRoomUser{
		RoomId: in.GetRoomId(),
		Accid:  in.GetAccid(),
	}
	if err = DelChatRoomUser(delRoomUser.RoomId, delRoomUser.Accid); err != nil {
		return
	}
	reply = &pb.RoomUserDelReply{}
	return
}
